// ComicSearch.java

/* Apisitt Rattana, Jan 14, 2001
   Computer Engineering, Prince of Songkla University
   Revised by
     Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001
*/
/*
  ComicSearch is a GUI interface in front of 5 threads.
  The GUI interface obtains a comic title and issue number
  and then searches for information about that comic.
  
  It contacts 4 search engines (using 4 threads):
    Nick Simon's Silver Age search engine
    AuctionHawk
    Google
    GCD
  It uses the fifth thread to search a local database of
  comic information.

  The results are displayed in a table, each row containing
  a Web page title, its URL, and a source ID for where the
  information comes from. The user can click on the title or
  URL to display the page in the default Web browser. The
  user can also click on the source ID to go to the home
  page of the source.

*/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.TableModelEvent;


public class ComicSearch extends JFrame
						implements ActionListener
{
  // GUI
  private JButton Search, SaveButton, Stop, Clear, About;
  private JTextField ssRun, ggRun, dbRun, gcdRun, ahRun;
  private JTextField issue_title, issue_no;
  private JComboBox sourcesList;
  private Container c;

  private JTable resTable;       
  private ResultTModel tableData;   // data model for the table

  // Encrypted userID:password for proxy authentication
  private String enIDPass = null;

  // flag monitored by the search threads, so they know if to stop
  private boolean stopSearch = false;   

  // flags set by the search threads to indicate they have finished
  private boolean ssFinish = false, ggFinish = false, dbFinish = false,
                  gcdFinish = false, ahFinish = false;

  // Titles database and SourceMap
  private Titles ts;
  private SourceMap sm;      // maps source ID to name/URL info
  

  public ComicSearch(String enIP)
  {
    super( "ComicSearch 1.0" );

	ts = new Titles("titlesDB.txt");      // load the Titles database
	sm = ts.getSourceMap();

    enIDPass = enIP;              // save the encrypted auth. password
    c = getContentPane();
    JPanel panel  = new JPanel();

    panel.add(new JLabel("Issue "));       // issue title input
    issue_title = new JTextField("", 10);
    panel.add(issue_title);

    panel.add(new JLabel("No. "));         // issue number input
    issue_no = new JTextField("1", 3);
    panel.add(issue_no);
    
    Search = new JButton("Start Search");
    panel.add(Search);
    Search.addActionListener( this );

    Stop = new JButton("Stop Search");
    Stop.setCursor (Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    panel.add(Stop);
    Stop.addActionListener( this );

    Clear = new JButton("Reset");
    Clear.setCursor (Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    panel.add(Clear);
    Clear.addActionListener( this );

	// list of sources
	sourcesList = new JComboBox( sm.getSourceIDs() );
    sourcesList.setCursor (Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	sourcesList.insertItemAt("Sources", 0);
	sourcesList.setSelectedIndex(0);
	sourcesList.setEditable(false);
	sourcesList.addActionListener(this);
    panel.add(sourcesList);

    // bar showing status of search threads
    displayRunBar(panel);

    About = new JButton("?");
    About.setCursor (Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    panel.add(About);
    About.addActionListener( this );


    c.add( panel, BorderLayout.NORTH );

    displayTable(c);

    setSize(800, 410);
    show();
  } // end of ComicSearch()


  private void displayRunBar(JPanel panel)
  // The runbar is a series of coloured textfields for the search engines,
  // gray when they are inactive; green when running
  {
     ssRun = new JTextField("SS");
     ssRun.setEditable(false);
     ssRun.setBackground(Color.lightGray);
     ssRun.setToolTipText("Executing Searches (shown in green)");

     ggRun = new JTextField("GG");
     ggRun.setEditable(false);
     ggRun.setBackground(Color.lightGray);

     dbRun = new JTextField("DB");
     dbRun.setEditable(false);
     dbRun.setBackground(Color.lightGray);
     dbRun.setToolTipText("Executing Searches (shown in green)");

     gcdRun = new JTextField("GCD");
     gcdRun.setEditable(false);
     gcdRun.setBackground(Color.lightGray);

     ahRun = new JTextField("AH");
     ahRun.setEditable(false);
     ahRun.setBackground(Color.lightGray);
     ahRun.setToolTipText("Executing Searches (shown in green)");

     JPanel p1 = new JPanel();
     p1.setLayout( new BoxLayout(p1, BoxLayout.X_AXIS));
     p1.add(ssRun); p1.add(ggRun);
     p1.add(dbRun); p1.add(gcdRun); p1.add(ahRun);

     panel.add(p1);

  }  // end of displayRunBar()


  private void displayTable(Container c)
  // the results table: each row contains
  //         row no, web page title, URL, source ID
  {
    tableData = new ResultTModel();

    resTable = new JTable(tableData);
    // resTable.setAutoCreateColumnsFromModel(false);
    resTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    TableColumn column = null;
    for (int i = 0; i < ResultTModel.columns.length; i++) {
      column = resTable.getColumnModel().getColumn(i);
      column.setPreferredWidth( ResultTModel.columns[i].width );
    }

    resTable.getTableHeader().setReorderingAllowed(false);
    resTable.setPreferredScrollableViewportSize(new Dimension(500, 150));
    JScrollPane sp = new JScrollPane(resTable);
    c.add(sp, BorderLayout.CENTER);

    // let a user click inside the table
    resTable.addMouseListener(new MouseAdapter() {
         public void mouseClicked(MouseEvent e) 
         {  openURL(resTable);  }
    });

  } // end of displayTable()


  // Open the default browser to display a URL
  private void openURL(JTable table)
  {
    int numRows = table.getRowCount();
    int numCols = table.getColumnCount();
    TableModel model = table.getModel();
	String urlText;

    int col, row;
    if ((col = table.getSelectedColumn()) != -1 && 
        (row = table.getSelectedRow()) != -1) {
      if ((col == 1)||(col == 2)) {      // anchor or URL field
	    urlText = model.getValueAt(row,2).toString();
	    BrowserControl.displayURL( urlText );     // show URL in browser
	  }
	  else if (col == 3) {              // sourceID field
	    String sID = model.getValueAt(row,3).toString();
		urlText = sm.getSourceURL( sID );     // get source URL
	    BrowserControl.displayURL( urlText );
	  }
    } 
    else    // Display error
      JOptionPane.showMessageDialog(null, 
	       "No row selected -- left click first",
           "Warning", JOptionPane.WARNING_MESSAGE);
  } // end of openURL()


  public void actionPerformed( ActionEvent e )
  { 
     Object s = e.getSource();

     if (s == Search) {
       stopSearch = false;
       startSearch();
     } 
     else if (s == Stop) {
      stopSearch = true;     // this flag is monitored by the search threads
      c.setCursor (Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    } 
    else if (s == Clear) {
      clearRows();
      issue_title.setText("");
      issue_no.setText("1");
      issue_title.requestFocus();
    }
	else if (s == sourcesList)
      selectSource( (JComboBox)s );
    else if (s == About)
      showAboutDetails();
  }

  
  private void showAboutDetails()
  {
    String msg = "ComicSearch 1.0    (March 2001)\n\n" +
         "  Apisitt Rattana and Andrew Davison\n" +
         "  Computer Engineering, Prince of Songkla University\n" +
         "  Hat Yai, Songkhla, Thailand\n" +
         "  E-mail: dandrew@ratree.psu.ac.th\n\n" +
         "ComicSearch searches for comic covers and information\n" +
         "using Google, AuctionHawk, GCD, Nick Simon's Silver Age\n" +
         "search engine, and a database of good sources (including\n" +
         "Two Tub Man's site and Mike's Amazing World of DC site).\n\n" +
         "Quick start\n" +
         "  1. Type in a title and issue number, then\n" +
         "     press \"Start Search\".\n" +
         "  2. Select a row from the results table, then\n" +
         "     click on the title or URL to load the image\n" +
         "     into your local browser.\n\n" +
         "More Info.\n" +
         "  For information on using ComicSearch with proxies/\n" +
         "  firewalls, extending the search database, etc., please\n" +
         "  read readme.html in the ComicSearch directory.";

    JTextArea jt = new JTextArea(msg, 25, 30);
    jt.setEditable(false);
    JOptionPane.showMessageDialog(null, jt, 
       "About ComicSearch", JOptionPane.INFORMATION_MESSAGE);
  } // end of aboutDetails()


  public void startSearch()
  // check the title & issue number, then start the threads
  {
    String Ctitle = issue_title.getText().trim();
    String Cissue = issue_no.getText().trim();

    if (Ctitle.equals(""))
      JOptionPane.showMessageDialog(null, 
	       "You did not enter a comics title",
           "Warning", JOptionPane.WARNING_MESSAGE);
    else if (Cissue.equals(""))
      JOptionPane.showMessageDialog(null, 
	        "You did not enter a issue number",
            "Warning", JOptionPane.WARNING_MESSAGE);				
    else {
      try {
         int inum = Integer.parseInt( Cissue );
         if (inum < 0)
                 JOptionPane.showMessageDialog(null, 
	       "You must enter a positive issue number",
           "Warning", JOptionPane.WARNING_MESSAGE);
         else {
           String issue = "" + inum;      // leading 0's are gone 
           clearRows();
           c.setCursor (Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
           startThreads(Ctitle, issue, enIDPass);
         }
       }
       catch( NumberFormatException e) {
         JOptionPane.showMessageDialog(null, 
	       "You must enter a numeric issue number",
           "Warning", JOptionPane.WARNING_MESSAGE);
       }
    } // else
  } // end of startSearch()


  private void startThreads(String Ctitle, String Cissue, String enIDpass)
  {
    // Silver Age Search
    ssFinish = false;    // thread 'finished running' flags
    ssRun.setBackground(Color.green);
	new SilverThread(Ctitle, Cissue, enIDpass, this).start();

    // AuctionHawk Search
    ahFinish = false;
    ahRun.setBackground(Color.green);
    new AHThread(Ctitle, Cissue, enIDpass, this).start();

    // Google Search
    ggFinish = false;
    ggRun.setBackground(Color.green);
    new GGThread(Ctitle, Cissue, enIDpass, this).start();

    // GCD Search
    gcdFinish = false;
    gcdRun.setBackground(Color.green);
    new GCDThread(Ctitle, Cissue, enIDpass, this).start();

	// database search using ts titles database
    dbFinish = false;
    dbRun.setBackground(Color.green);
    new DBThread(Ctitle, Cissue, ts, this).start();

  } // end of startThreads()


  synchronized public void writeRow(SearchResults sr)
  // Used by the threads to write a new row into the display table
  { 
    int row = resTable.getRowCount();
    tableData.insert(row+1, sr);        // insert into table

    // Tell listeners the row changed.
    resTable.tableChanged( new TableModelEvent(
        tableData, row+1, row+1,
        TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));

     resTable.repaint();       // redraw result table
  } // end of writeRow()



  // Clear all the rows of the table
  private void clearRows()
  { 
    int numRows = tableData.getRowCount();
    if (numRows > 0) {
      tableData.clear();
      // tell listeners the rows have changed
      resTable.tableChanged( new TableModelEvent(
         tableData, 0, numRows-1, 
		 TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
      resTable.repaint();
    }
  }  // end of clearRows()


  synchronized public void threadFinished(String threadLabel)
  // used by the threads to signal that they have finished
  {
    if (threadLabel.equals("SS")) {
      ssFinish = true;
      ssRun.setBackground(Color.lightGray);
    }
    else if (threadLabel.equals("GG")) {
      ggFinish = true;
      ggRun.setBackground(Color.lightGray);
    }
    else if (threadLabel.equals("DB")) {
      dbFinish = true;
      dbRun.setBackground(Color.lightGray);
    }
    else if (threadLabel.equals("GCD")) {
      gcdFinish = true;
      gcdRun.setBackground(Color.lightGray);
    }
    else if (threadLabel.equals("AH")) {
      ahFinish = true;
      ahRun.setBackground(Color.lightGray);
    }

    if ( ssFinish && ggFinish && dbFinish && gcdFinish  && ahFinish ) {
      c.setCursor (Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
  } // end of threadFinished()


  public boolean stopSearchPressed()
  {  return (stopSearch == true);  }


  private void selectSource(JComboBox jc)
  // display the source name and URL for a given ID selected from the combo box
  {
	String sourceID = (String) jc.getSelectedItem();
	String sourceName = sm.getSourceName( sourceID );
	String sourceURL = sm.getSourceURL( sourceID );
    if (sourceName != null)
	  JOptionPane.showMessageDialog(null, 
	        "Source " + sourceID + " is \n" + sourceName + "\n" + sourceURL,
			"Source Information",
			JOptionPane.INFORMATION_MESSAGE);
	sourcesList.setSelectedIndex(0);     // so "Source" label is always visible
  }


  public static void main ( String[] args)
  {
    ProxyDetails pd = new ProxyDetails(args);
    String enIDPass = pd.getEnIDPass();
    // System.out.println("Encrypted IDPass: " + enIDPass );

    ComicSearch app = new ComicSearch(enIDPass);

	app.addWindowListener( new WindowAdapter() {
      public void windowClosing ( WindowEvent e )
      { System.exit(0);  }
	});
  }

}  // end of ComicSearch class


